module height
  use prm, only : IMAX, JMAX, DX, dt
  use Tbound, only: Ms

  implicit none

  private 
  public get_H
  public add_Ms

contains

  subroutine add_Ms(Hsfc1, Hbnd, Hsfc2)
    implicit none
    real(8), INTENT(IN)   :: Hsfc1(1:IMAX, 1:JMAX)
    real(8), INTENT(IN)   :: Hbnd(1:IMAX, 1:JMAX)
    real(8), INTENT(OUT)  :: Hsfc2(1:IMAX, 1:JMAX)
    integer  :: i, j    


    do j = 1, JMAX
       Hsfc2(1, j) = 0.D0
       Hsfc2(IMAX, j) =0.D0
       do i = 2, IMAX-1
          Hsfc2(i, j)= Hsfc1(i, j) &
               & + Hbnd(i, j) + Ms(i, j) * dt
          if(Hsfc2(i, j) <= 0.0D0)then
             Hsfc2(i, j) = 0.0D0
          end if
       end do
    end do


  end subroutine add_Ms

  subroutine get_H(Hsfc1, Hbnd, Ufl, Hsfc2)
    implicit none
    real(8), INTENT(IN)   :: Hsfc1(1:IMAX, 1:JMAX)
    real(8), INTENT(IN)   :: Hbnd(1:IMAX, 1:JMAX)
    real(8), INTENT(IN)   :: Ufl(0:IMAX, 1:JMAX)
    real(8), INTENT(OUT)  :: Hsfc2(1:IMAX, 1:JMAX)

    integer  :: i, j
  
    do j = 1, JMAX
       Hsfc2(1, j) = 0.D0
       Hsfc2(IMAX, j) =0.D0
       do i = 2, IMAX-1
          Hsfc2(i, j)= Hsfc1(i, j) &
               & + Hbnd(i, j) - dt/dx*(Ufl(i, j) - Ufl(i-1, j)) + Ms(i, j) * dt
          if(Hsfc2(i, j) <= 0.0D0)then
             Hsfc2(i, j) = 0.0D0
          end if
       end do
    end do

!    $BI9>2$NC<N.$l=P$5$J$$LdBj$N%A%'%C%/(B
!    do j = 1, JMAX
!       do i = IMAX, 1, -1
!          if(Hsfc1(i, j)>0D0)then
!             write(6, *)'Hsfc2=', Hsfc2(i-1, j), Hsfc2(i, j), Hsfc2(i+1, j)
!             write(6, *)'Hsfc1=',Hsfc1(i-1, j), Hsfc1(i, j), Hsfc1(i+1, j)
!             write(6, *)'Ufl(i)=', Ufl(i-1, j),  Ufl(i, j), Ufl(i+1, j)
!             write(6, *)'Ufl(i-1)=' , Ufl(i-2, j), Ufl(i-1, j), Ufl(i, j)
!             write(6, *)'-dt*(Ufl(i, j) - Ufl(i-1, j))/dx=', &
!                  & - dt*(Ufl(i-1, j) - Ufl(i-2, j))/dx, &
!                  & - dt*(Ufl(i, j) - Ufl(i-1, j))/dx, &
!                  & - dt*(Ufl(i+1, j) - Ufl(i, j))/dx
!             exit
!          end if
!
!       end do
!    end do


end subroutine get_H


end module height
