!=====================================================================
! $B29EYJ,I[7W;;%b%8%e!<%k(B
!  pending
!   - x = 1 and IMAX $B$N%(%M%k%.!<N.F~NL$O%+%&%s%H$7$F$J$$(B
! 2003/12/16 
! 2004/01/01  energy conservation $B$r9MN8$7$?9M$(J}$KJQ99(B
!=====================================================================
module T_pack
  
  use prm, only : IMAX, JMAX, KMAX,dt, dx
  
  use prmphys, only: k_ice, gamma, rho, c_p
  use com_var, only: zk, wgt, zk_half
  use Tbound,  only: Ts, Ms


  implicit none
  private
  
  public get_T
  public mk_energy
  public add_energy_Ms
  public add_energy_F
  public adjust_energy_Ms

contains

!=====================================================================
! $B3F3J;R$N%(%M%k%.!<$N<0(B
!=====================================================================
subroutine mk_energy(Temp1, energy)
  implicit none
  
  real(8), intent(IN)  :: Temp1(1:IMAX, 1:JMAX, 1:KMAX)
  real(8), intent(OUT) :: energy(1:IMAX, 1:JMAX, 1:KMAX)

  energy =  Temp1 * rho  * c_p

end subroutine mk_energy


!=====================================================================
subroutine get_T(Hsfc1,Temp1, Temp2)
  implicit none

  real(8), INTENT(IN)  :: Hsfc1(1:IMAX, 1:JMAX)
  real(8), INTENT(IN)  :: Temp1(1:IMAX, 1:JMAX, 1:KMAX)

  real(8), INTENT(OUT) :: Temp2(1:IMAX, 1:JMAX, 1:KMAX)


  real(8)              :: kappa                           
  real(8)              :: Tbnd(1:IMAX, 1:JMAX)            ! $B:G2<AX$N2<$N29EY(B

  real(8)              :: WORK_aft(1:IMAX, 1:JMAX, 1:KMAX)! $B78?t(B k+1
  real(8)              :: WORK_cnt(1:IMAX, 1:JMAX, 1:KMAX)! $B78?t(B k 
  real(8)              :: WORK_bef(1:IMAX, 1:JMAX, 1:KMAX)! $B78?t(B k-1

  integer              :: i, j, k

  kappa = k_ice/(rho*c_p)


  ! $B>eIt6-3&CM$OI=LL29EY(B
  Temp2(:, :, KMAX) = Ts
  ! $B2<It6-3&CM$OCO3LG.N.NLG.%U%i%C%/%9(B
  Tbnd(:, :) = Temp1(:, :, 1)+wgt(1)/k_ice * gamma



  do j = 1, JMAX
     do i = 1, IMAX
        ! $BI9>2$,$J$$>l9g$OI9>229EY(B
        if(Hsfc1(i, j) == 0.0)then
           Temp2(i, j, :) = Ts(i, j)
           cycle
        end if

        ! $B%l%Y%k(B 1 $B!A(B KMAX-1 $B$^$G$N78?t(B
        do k = 1, KMAX-1
           WORK_aft(i, j, k) =  kappa *dt  &
                &                /Hsfc1(i, j)**2/wgt(k)/(zk(k+1) - zk(k))
        end do
      
        WORK_cnt(i, j, 1) = 1 -  kappa *dt  &
             &                /Hsfc1(i, j)**2/wgt(1) &
             &              *(1/(zk(2) - zk(1)) + 1/wgt(1))
        do k = 2, KMAX-1
           WORK_cnt(i, j, k) = 1 -  kappa *dt  &
                &                /Hsfc1(i, j)**2/wgt(k) &
                &              *(1/(zk(k+1) - zk(k)) + 1/(zk(k) - zk(k-1)))
        end do
        
        WORK_bef(i, j, 1) =  kappa *dt   &
             &                /Hsfc1(i, j)**2/wgt(1)/wgt(1)
        do k=2, KMAX
           WORK_bef(i, j, k) =  kappa *dt   &
                &                /Hsfc1(i, j)**2/wgt(k)/(zk(k) - zk(k-1))
        end do



        ! $B3H;69`$N7W;;(B
        Temp2(i, j, 1) = Temp1(i, j, 1) *WORK_cnt(i, j, 1) &
             &           + Tbnd(i, j)   *WORK_bef(i, j, 1) &
             &           + Temp1(i, j, 2)*WORK_aft(i, j, 1) 
        do k=2, KMAX-1
           Temp2(i, j, k) = Temp1(i, j, k)*WORK_cnt(i, j, k) &
                &           + Temp1(i, j, k-1)*WORK_bef(i, j, k) &
                &           + Temp1(i, j, k+1)*WORK_aft(i, j, k) 
        end do
     end do
  end do

  

end subroutine get_T

!=====================================================================
! $BI=LL<};Y$K$h$k%(%M%k%.!<4sM?(B
!=====================================================================
subroutine add_energy_Ms(energy)
  implicit none

  real(8), intent(INOUT)  :: energy(1:IMAX, 1:JMAX, 1:KMAX)

  integer                :: i, j
  
  do j =1, JMAX
     do i = 1, IMAX
        energy(i, j, KMAX) = energy(i, j, KMAX) &
             &                  + Ts(i, j) * rho *c_p * Ms(i, j) * dx
!        write(6, *) i, j, 'energy(KMAX)=', energy(i, j, KMAX)
     end do
  end do

end subroutine add_energy_Ms

!=====================================================================
! $B?eJ?%(%M%k%.!<4sM?(B
!=====================================================================
subroutine add_energy_F(Temp1,  Uflgrd, Hsfc2, energy, energy2)
  implicit none

  real(8), intent(IN)     :: Uflgrd(   0:IMAX, 1:JMAX, 1:KMAX)   ! m^2/s
  real(8), intent(IN)     :: Temp1(1:IMAX, 1:JMAX, 1:KMAX)       ! K
  real(8), intent(IN)     :: Hsfc2(1:IMAX, 1:JMAX)               ! m
  real(8), intent(INOUT)  :: energy(1:IMAX, 1:JMAX, 1:KMAX)      ! J/m
  real(8), intent(OUT)    :: energy2(1:IMAX, 1:JMAX, 1:KMAX)      ! J/m

  real(8)                 :: Uflgrdl_pls(0:IMAX, 1:JMAX, 1:KMAX) !m^2/s
  real(8)                 :: Uflgrdl_mns(0:IMAX, 1:JMAX, 1:KMAX) !m^2/s
  real(8)                 :: Uflgrdr_pls(0:IMAX, 1:JMAX, 1:KMAX) !m^2/s
  real(8)                 :: Uflgrdr_mns(0:IMAX, 1:JMAX, 1:KMAX) !m^2/s


  integer                 :: i, j, k

  ! x $BJ}8~6-3&(B
  energy(1, :, :)    = 0.0D0
  energy(IMAX, :, :) = 0.0D0
  
  do j =1, JMAX
     do i = 2, IMAX-1
        do k = 1, KMAX
           ! $B<ANL%U%i%C%/%9$NId9f$rJ,$1$k(B
           ! $B3J;R(B i $B$N:8B&(B
           if(Uflgrd(i-1, j, k) >= 0)then ! $B3J;R(B i$B$XN.F~(B
              Uflgrdl_pls(i, j, k) = Uflgrd(i-1, j, k) !* Temp1(i-1, j, k)*c_p
              Uflgrdl_mns(i, j, k) = 0.0D0
           else ! $BN.=P(B
              Uflgrdl_pls(i, j, k) = 0.0D0
              Uflgrdl_mns(i, j, k) = Uflgrd(i-1, j, k) !* Temp1(i, j, k)*c_p
             
           end if

           ! $B3J;R(B i $B$N1&B&(B
           if(Uflgrd(i, j, k) >= 0)then ! $BN.=P(B
              Uflgrdr_pls(i, j, k) = 0.0D0
              Uflgrdr_mns(i, j, k) = - Uflgrd(i, j, k) ! * Temp1(i, j, k)*c_p
           else ! $BN.F~(B
              Uflgrdr_pls(i, j, k) = - Uflgrd(i, j, k) ! * Temp1(i+1, j, k)c_p
              Uflgrdr_mns(i, j, k) = 0.0D0
           end if
           
           energy(i, j, k) = energy(i, j, k) &
                &           + (Uflgrdl_pls(i, j, k)+Uflgrdl_mns(i, j, k))   &
                &             * dt * rho & 
                &           + (Uflgrdr_pls(i, j, k)+Uflgrdr_mns(i, j, k))   &
                &             * dt * rho 

           energy2(i, j, k) = Hsfc2(i, j) * wgt(k) * rho *dx
        end do


     end do
  end do

end subroutine add_energy_F


!=====================================================================
subroutine adjust_energy_Ms(Temp1, Hsfc1, Hsfc2, energy)
  implicit none

  real(8), intent(IN)    :: Temp1(1:IMAX, 1:JMAX, 1:KMAX)
  real(8), intent(in)    :: Hsfc1(1:IMAX, 1:JMAX)
  real(8), intent(in)    :: Hsfc2(1:IMAX, 1:JMAX)
  real(8), intent(inout) :: energy(1:IMAX, 1:JMAX, 1:KMAX)

 
  real(8)                :: comp_Ms(1:IMAX, 1:JMAX)   ! $B$+$sM\$K$h$kI9$N<ANL(B
  real(8)                :: comp_ice(1:IMAX, 1:JMAX, 1:KMAX) ! $B85$NI9$N<ANL(B

  real(8)                :: comp_iceup(1:IMAX, 1:JMAX, 1:KMAX)!$B2<$X0\F0$7$?I9NL(B
  real(8)                :: comp_icedn(1:IMAX, 1:JMAX, 1:KMAX)!$B85$+$i$"$kI9NL(B
  real(8)                :: col_mass(1:IMAX, 1:JMAX)  ! $B%+%i%`Cf$N<ANL(B
  real(8)                :: mass_Ms(1:IMAX, 1:JMAX)   ! $B$+$sM\$N<ANL(B
!  real(8)                :: Mass(1:IMAX, 1:JMAX)   ! $B$+$sM\$N<ANL(B

  integer                :: i, j, k 
! $B%+%i%`Cf$N<ANL(B
!  col_mass(:, :) = Hsfc1(:, :) * dx + Ms(:, :)*dt*dx 
  col_mass(:, :) = Hsfc2(:, :) * dx
! $B$+$sM\$N<ANL(B
  mass_Ms( :, :) = Ms(:, :)*dt*dx

  do j = 1,JMAX
     do i = 1, IMAX

        do k = KMAX, 1, -1

           ! $B$+$sM\$K$h$kI9$H85$+$i$"$kI9$,$"$k3J;R(B
           if(zk_half(k-1) <= 1.0D0 - mass_Ms(i, j)/col_mass(i, j))then
           ! $B$+$sM\$K$h$kI9(B
              comp_Ms(i, j) =Ms(i, j) * rho * dt * dx  &
                   & - Hsfc2(i, j) * (1.0D0 - zk_half(k)) * rho * dx 
           ! $B85$+$i$"$k$NI9(B($B?7$7$$3J;R%5%$%:(B - $B$+$sM\$K$h$C$FA}$($?I9(B)
              comp_ice(i, j, k) = Hsfc2(i, j)*wgt(k) * dx * rho - comp_Ms(i, j)
              comp_iceup(i, j, k) = comp_Ms(i, j)
              comp_icedn(i, j, k) = comp_ice(i, j, k)

              energy(i, j, k) = comp_iceup(i, j, k) &!* c_p * Ts(i,j) &
                   &         + comp_icedn(i, j, k) !* c_p * Temp1(i, j, k)
              
             ! k-1 $B0J2<$NI9$N3d9g(B 
              comp_iceup(i, j, 1:k-1) = comp_Ms(i, j)
              comp_icedn(i, j, 1:k-1) = Hsfc2(i, j)*wgt(1:k-1)*dx*rho &
                   &                 - comp_ms(i, j)

              ! $B;~9o(B t + 1 $B$G$N%(%M%k%.!<(B
              energy(i, j, 1:k-1) = &
                   &     comp_iceup(i, j, 1:k-1)&! * c_p * Temp1(i, j, 2:k)  &
                   &      + comp_icedn(i, j, 1:k-1)!* c_p * Temp1(i, j, 1:k-1)
   

              ! $B;~9o(B t + 1 $B$G$N<ANL(B 
              comp_ice(i, j, 1:k-1) =  comp_iceup(i, j, 1:k-1) &
                   &                  + comp_icedn(i, j, 1:k-1) 
              exit 

           else
              ! $B$+$sM\$K$h$k%(%M%k%.!<$NA}2C(B
              comp_ice(i, j, k) = Hsfc2(i, j)*wgt(k)*rho*dx 
              energy(i, j, k) = comp_ice(i, j, k) !* c_p * Ts(i, j)
           end if
        end do
        ! $B;~9o(B t $B$GI9$,$J$$>l9g(B
        if((Hsfc1(i, j) == 0.0D0).and.(Hsfc2(i, j) /= 0.0D0))then
           write(6, *) 'growing ice in position', i, j
           comp_ice(i, j, :) = Hsfc2(i, j)*wgt(:) *dx*rho
           comp_Ms(i, j) = 0.0D0
           comp_iceup(i, j, :) = 0.0D0
           comp_icedn(i, j, :) = 0.0D0
           energy(i, j, :) = comp_ice(i, j, :) !* c_p * Ts(i, j)
        end if


!        if((j == 2).and.(i==2))then
!           write(6, *)'k=', i, j,  k
!           write(6, *)'comp_iceup', i, REAL(comp_iceup(i,2,:))/rho/dx
!           write(6, *)'comp_icedn', i, REAL(comp_icedn(i,2,:))/rho/dx
!           write(6, *)'comp_Ms'   , i, REAL(comp_Ms(i,2))/rho/dx
!           write(6, *)'comp_ice', i, REAL(comp_ice(i,2,:))/rho/dx
!           write(6, *)'Hsfc1', i, Hsfc1(i, j), REAL(Hsfc1(i, j)*wgt(:))
!           write(6, *)'Hsfc2', i, Hsfc2(i, j), REAL(Hsfc2(i, j)*wgt(:))

!           Mass = 0.0D0
!           do k=1, KMAX
!              Mass(i, j)  = Mass(i, j) + energy(i, j, k)
!           end do
!           write(6, *)'Mass=',  Mass(i, j)/rho/dx
!        end if
        
     end do
  end do

end subroutine adjust_energy_Ms

end module T_pack
